PTI has a function to clone page-table entries but only from the
init_mm page-table. Provide a new function to clone page-table
entries from a specified mm page-table.

Signed-off-by: Alexandre Chartre <alexandre.char...@oracle.com>
---
 arch/x86/include/asm/pti.h | 10 ++++++++++
 arch/x86/mm/pti.c          | 32 ++++++++++++++++----------------
 2 files changed, 26 insertions(+), 16 deletions(-)

diff --git a/arch/x86/include/asm/pti.h b/arch/x86/include/asm/pti.h
index 07375b476c4f..5484e69ff8d3 100644
--- a/arch/x86/include/asm/pti.h
+++ b/arch/x86/include/asm/pti.h
@@ -4,9 +4,19 @@
 #ifndef __ASSEMBLY__
 
 #ifdef CONFIG_PAGE_TABLE_ISOLATION
+
+enum pti_clone_level {
+       PTI_CLONE_PMD,
+       PTI_CLONE_PTE,
+};
+
+struct mm_struct;
+
 extern void pti_init(void);
 extern void pti_check_boottime_disable(void);
 extern void pti_finalize(void);
+extern void pti_clone_pgtable(struct mm_struct *mm, unsigned long start,
+                             unsigned long end, enum pti_clone_level level);
 #else
 static inline void pti_check_boottime_disable(void) { }
 #endif
diff --git a/arch/x86/mm/pti.c b/arch/x86/mm/pti.c
index 1aab92930569..ebc8cd2f1cd8 100644
--- a/arch/x86/mm/pti.c
+++ b/arch/x86/mm/pti.c
@@ -294,14 +294,8 @@ static void __init pti_setup_vsyscall(void)
 static void __init pti_setup_vsyscall(void) { }
 #endif
 
-enum pti_clone_level {
-       PTI_CLONE_PMD,
-       PTI_CLONE_PTE,
-};
-
-static void
-pti_clone_pgtable(unsigned long start, unsigned long end,
-                 enum pti_clone_level level)
+void pti_clone_pgtable(struct mm_struct *mm, unsigned long start,
+                      unsigned long end, enum pti_clone_level level)
 {
        unsigned long addr;
 
@@ -320,7 +314,7 @@ pti_clone_pgtable(unsigned long start, unsigned long end,
                if (addr < start)
                        break;
 
-               pgd = pgd_offset_k(addr);
+               pgd = pgd_offset(mm, addr);
                if (WARN_ON(pgd_none(*pgd)))
                        return;
                p4d = p4d_offset(pgd, addr);
@@ -409,6 +403,12 @@ pti_clone_pgtable(unsigned long start, unsigned long end,
        }
 }
 
+static void pti_clone_init_pgtable(unsigned long start, unsigned long end,
+                                  enum pti_clone_level level)
+{
+       pti_clone_pgtable(&init_mm, start, end, level);
+}
+
 #ifdef CONFIG_X86_64
 /*
  * Clone a single p4d (i.e. a top-level entry on 4-level systems and a
@@ -476,7 +476,7 @@ static void __init pti_clone_user_shared(void)
        start = CPU_ENTRY_AREA_BASE;
        end   = start + (PAGE_SIZE * CPU_ENTRY_AREA_PAGES);
 
-       pti_clone_pgtable(start, end, PTI_CLONE_PMD);
+       pti_clone_init_pgtable(start, end, PTI_CLONE_PMD);
 }
 #endif /* CONFIG_X86_64 */
 
@@ -495,9 +495,9 @@ static void __init pti_setup_espfix64(void)
  */
 static void pti_clone_entry_text(void)
 {
-       pti_clone_pgtable((unsigned long) __entry_text_start,
-                         (unsigned long) __entry_text_end,
-                         PTI_CLONE_PMD);
+       pti_clone_init_pgtable((unsigned long) __entry_text_start,
+                              (unsigned long) __entry_text_end,
+                              PTI_CLONE_PMD);
 }
 
 /*
@@ -572,11 +572,11 @@ static void pti_clone_kernel_text(void)
         * pti_set_kernel_image_nonglobal() did to clear the
         * global bit.
         */
-       pti_clone_pgtable(start, end_clone, PTI_LEVEL_KERNEL_IMAGE);
+       pti_clone_init_pgtable(start, end_clone, PTI_LEVEL_KERNEL_IMAGE);
 
        /*
         * pti_clone_pgtable() will set the global bit in any PMDs
         * that it clones, but we also need to get any PTEs in
         * the last level for areas that are not huge-page-aligned.
         */
 
@@ -572,11 +572,11 @@ static void pti_clone_kernel_text(void)
 
        /*
-        * pti_clone_pgtable() will set the global bit in any PMDs
-        * that it clones, but we also need to get any PTEs in
+        * pti_clone_init_pgtable() will set the global bit in any
+        * PMDs that it clones, but we also need to get any PTEs in
         * the last level for areas that are not huge-page-aligned.
         */
-- 
2.18.4

Reply via email to